@@ -35,6 +35,24 @@
 static char sccsid[] = "@(#)ftp.c	5.38 (Berkeley) 4/22/91";
 #endif /* not lint */
 
+#if defined(SOCKS)
+#define connect	Rconnect
+#define getsockname	Rgetsockname
+#define listen	Rlisten
+#define accept	Raccept
+#define rcmd	Rrcmd
+# if defined(SHORTENED_RBIND)
+#define bind	Rbind
+# else /* SHORTENED_RBIND not defined */
+/*
+ * Be sure to put in socks_bind_remoteAddr (32-bit integer) the IP
+ * address (in network byte order) of the host it is aiming for,
+ * which is typically the destination host of the previous connect().
+ */
+#define bind(a,b,c)	Rbind(a,b,c,socks_bind_remoteAddr)
+# endif /* # if defined(SHORTENED_RBIND) */
+#endif /* #if defined(SOCKS) */
+
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -64,6 +82,10 @@ static char sccsid[] = "@(#)ftp.c	5.38 (Berkeley) 4/22/91";
 
 #include "ftp_var.h"
 
+#if defined(AIX) || defined(DGUX)
+#undef IP_TOS
+#endif
+
 #ifdef SOLARIS
 /* for bcopy(), bzero() and bcmp() */
 #include <string.h>
@@ -81,7 +103,7 @@ struct	sockaddr_in myctladdr;
 uid_t	getuid();
 sig_t	lostpeer();
 off_t	restart_point = 0;
-unsigned long remoteAddr;		/* Socks */
+unsigned long socks_bind_remoteAddr;		/* Socks */
 
 extern char *strerror();
 extern int connected, errno;
@@ -89,10 +111,10 @@ extern int connected, errno;
 FILE	*cin, *cout;
 FILE	*dataconn();
 
-#ifndef SVR3
+#if !defined(SVR3) && !defined(BIND_RESOLVER) && !defined(LINUX)
 
 /* >>> YDL */
-#if	!defined(BSD) || (BSD <= 43)
+#if	!defined(BSD) || (BSD - 43 <= 0)
 
 char	*h_errlist[] = {
 	"Error 0",
@@ -109,13 +131,13 @@ int h_errno;		/* In some version of SunOS this is necessary */
  * herror --
  *	print the error indicated by the h_errno value.
  */
-#ifdef LINUX
+#if defined(AIX_PS2)
 void herror(s)
       const char *s;
-#else
+#else /* #if AIX_PS2 not defined */
 herror(s)
       char *s;
-#endif
+#endif /* #if defined(AIX_PS2) */
 {
 	if (s && *s) {
 		fprintf(stderr, "%s: ", s);
@@ -130,10 +152,10 @@ herror(s)
 		fprintf(stderr, "%s\n", h_errlist[h_errno]);
 	}
 }
-#endif	/* !define(BSD) || (BSD <= 43) */
+#endif	/* !define(BSD) || (BSD - 43 <= 0) */
 /* <<< YDL */
 
-#endif
+#endif /* !defined(SVR3) && !defined(BIND_RESOLVER) && !defined(LINUX) */
 
 char *
 hookup(host, port)
@@ -171,7 +193,7 @@ hookup(host, port)
 		return (0);
 	}
 	hisctladdr.sin_port = port;
-	while (Rconnect(s, (struct sockaddr *)&hisctladdr, sizeof (hisctladdr)) < 0) {
+	while (connect(s, (struct sockaddr *)&hisctladdr, sizeof (hisctladdr)) < 0) {
 		if (hp && hp->h_addr_list[1]) {
 			int oerrno = errno;
 			extern char *inet_ntoa();
@@ -198,20 +220,20 @@ hookup(host, port)
 		code = -1;
 		goto bad;
 	}
-	remoteAddr = hisctladdr.sin_addr.s_addr;	 /* Socks */
+	socks_bind_remoteAddr = hisctladdr.sin_addr.s_addr;	 /* Socks */
 	len = sizeof (myctladdr);
-	if (Rgetsockname(s, (struct sockaddr *)&myctladdr, &len) < 0) {
+	if (getsockname(s, (struct sockaddr *)&myctladdr, &len) < 0) {
 		perror("ftp: getsockname");
 		code = -1;
 		goto bad;
 	}
-#ifdef IP_TOS
+#if defined(IP_TOS) && defined(IPTOS_LOWDELAY)
 	tos = IPTOS_LOWDELAY;
 	if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
 		perror("ftp: setsockopt TOS (ignored)");
 #endif
 	cin = fdopen(s, "r");
-#if defined(ISC) || defined(__alpha)
+#if defined(ISC) || defined(__alpha) || defined(SCO)
 	cout = fdopen(dup(s), "w");
 #else
 	cout = fdopen(s, "w");
@@ -939,7 +961,11 @@ recvrequest(cmd, local, remote, lmode, printnames)
 	case TYPE_I:
 	case TYPE_L:
 		if (restart_point &&
-		    lseek(fileno(fout), (long) restart_point, L_SET) < 0) {
+#if defined(SEEK_SET)
+		  lseek(fileno(fout), (long) restart_point, SEEK_SET) < 0) {
+#else
+		  lseek(fileno(fout), (long) restart_point, L_SET) < 0) {
+#endif
 			fprintf(stderr, "local: %s: %s\n", local,
 				strerror(errno));
 			if (closefunc != NULL)
@@ -983,7 +1009,11 @@ recvrequest(cmd, local, remote, lmode, printnames)
 		if (restart_point) {
 			register int i, n, ch;
 
+#if defined(SEEK_SET)
+			if (fseek(fout, 0L, SEEK_SET) < 0)
+#else
 			if (fseek(fout, 0L, L_SET) < 0)
+#endif
 				goto done;
 			n = restart_point;
 			for (i = 0; i++ < n;) {
@@ -992,10 +1022,10 @@ recvrequest(cmd, local, remote, lmode, printnames)
 				if (ch == '\n')
 					i++;
 			}
-#ifndef SOLARIS
-			if (fseek(fout, 0L, L_INCR) < 0) {
-#else
+#if defined(SEEK_CUR)
 			if (fseek(fout, 0L, SEEK_CUR) < 0) {
+#else
+			if (fseek(fout, 0L, L_INCR) < 0) {
 #endif
 done:
 				fprintf(stderr, "local: %s: %s\n", local,
@@ -1128,7 +1158,7 @@ noport:
 			perror("ftp: setsockopt (reuse address)");
 			goto bad;
 		}
-	if (Rbind(data, (struct sockaddr *)&data_addr, sizeof (data_addr), remoteAddr) < 0) {
+	if (bind(data, (struct sockaddr *)&data_addr, sizeof(data_addr)) < 0) {
 		perror("ftp: bind");
 		goto bad;
 	}
@@ -1136,11 +1166,11 @@ noport:
 	    setsockopt(data, SOL_SOCKET, SO_DEBUG, (char *)&on, sizeof (on)) < 0)
 		perror("ftp: setsockopt (ignored)");
 	len = sizeof (data_addr);
-	if (Rgetsockname(data, (struct sockaddr *)&data_addr, &len) < 0) {
+	if (getsockname(data, (struct sockaddr *)&data_addr, &len) < 0) {
 		perror("ftp: getsockname");
 		goto bad;
 	}
-	if (Rlisten(data, 1) < 0)
+	if (listen(data, 1) < 0)
 		perror("ftp: listen");
 	if (sendport) {
 		a = (char *)&data_addr.sin_addr;
@@ -1159,7 +1189,7 @@ noport:
 	}
 	if (tmpno)
 		sendport = 1;
-#ifdef IP_TOS
+#if defined(IP_TOS) && defined(IPTOS_THROUGHPUT)
 	on = IPTOS_THROUGHPUT;
 	if (setsockopt(data, IPPROTO_IP, IP_TOS, (char *)&on, sizeof(int)) < 0)
 		perror("ftp: setsockopt TOS (ignored)");
@@ -1179,7 +1209,7 @@ dataconn(lmode)
 	struct sockaddr_in from;
 	int s, fromlen = sizeof (from), tos;
 
-	s = Raccept(data, (struct sockaddr *) &from, &fromlen);
+	s = accept(data, (struct sockaddr *) &from, &fromlen);
 	if (s < 0) {
 		perror("ftp: accept");
 		(void) close(data), data = -1;
@@ -1187,7 +1217,7 @@ dataconn(lmode)
 	}
 	(void) close(data);
 	data = s;
-#ifdef IP_TOS
+#if defined(IP_TOS) && defined(IPTOS_THROUGHPUT)
 	tos = IPTOS_THROUGHPUT;
 	if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
 		perror("ftp: setsockopt TOS (ignored)");
