@@ -2125,7 +2125,7 @@ telnet(user)
 	int len = sizeof(local_host);
 
 	if (!local_host[0]) {
-		gethostname(local_host, &len);
+		gethostname(local_host, len);
 		local_host[sizeof(local_host)-1] = 0;
 	}
 	auth_encrypt_init(local_host, hostname, "TELNET", 0);
